import { h, HyperNode, normaliseParams } from "./node.js";
import { parseSelector } from "./parse.js";
import { isNonNullable } from "./util.js";
import { ReadonlyState, State } from "./state.js";
function createSelectorProxy(element, hyperElement, loaded) {
    return new Proxy(hyperElement, {
        // Do NOT cache the target _ like we do below in elements,
        // because if users use this feature a lot, it'll leak memory
        // Just let elements.a.hello be a new function every time
        // and elements.a.hello !== elements.a.hello
        // It's okay -- don't fret about it
        get(_, selector) {
            const parsed = parseSelector([loaded, selector].filter(Boolean).join(" "));
            const hyperElement = function hyperElement(props, ...childNodes) {
                const { attrs, children } = normaliseParams(props, childNodes);
                const className = new State("");
                if (ReadonlyState.isState(attrs.class)) {
                    className.set(attrs.class.value);
                    attrs.class.pipe(className);
                }
                else
                    className.set(attrs.class);
                if (parsed.class)
                    className.setWith(c => [parsed.class, c].flatMap(x => (x ? x : [])).filter(Boolean));
                const merged = { ...attrs, id: parsed.id || attrs.id, class: className };
                return new HyperNode(element, merged, children.filter(isNonNullable));
            };
            return createSelectorProxy(element, hyperElement, selector);
        },
    });
}
export function create(element) {
    const hyperElement = function hyperElement(...params) {
        return h(element, ...params);
    };
    return /* @__PURE__ */ createSelectorProxy(element, hyperElement);
}
