import { ReadonlyState, State } from "./state.ts";
export declare class Member<T> extends State<T> {
    #private;
    parent: List<T>;
    constructor({ parent, value }: {
        parent: List<T>;
        value: T;
    });
    getCurrentIndex(): number;
    remove(): void;
    replace(value: T): void;
    readonly(): ReadonlyMember<T>;
}
export declare class ReadonlyMember<T> extends ReadonlyState<T> {
    parent: ReadonlyList<T>;
    constructor(member: Member<T>, parent?: ReadonlyList<T>);
}
export declare enum ListEventKind {
    Append = "append",
    Prepend = "prepend",
    Insert = "insert",
    Remove = "remove",
    Replace = "replace",
    Swap = "swap",
    Move = "move",
    Update = "update",
    MemberUpdate = "member-update"
}
type ListEvent = {
    kind: ListEventKind.Append;
    list: List<any>;
    member: Member<any>;
} | {
    kind: ListEventKind.Prepend;
    list: List<any>;
    member: Member<any>;
} | {
    kind: ListEventKind.Insert;
    list: List<any>;
    member: Member<any>;
    index: number;
} | {
    kind: ListEventKind.Remove;
    list: List<any>;
    member: Member<any>;
    index: number;
} | {
    kind: ListEventKind.Replace;
    list: List<any>;
    member: Member<any>;
    old: Member<any>;
    index: number;
} | {
    kind: ListEventKind.Swap;
    list: List<any>;
    a: Member<any>;
    b: Member<any>;
    from: number;
    to: number;
} | {
    kind: ListEventKind.Move;
    list: List<any>;
    member: Member<any>;
    from: number;
    to: number;
} | {
    kind: ListEventKind.Update;
    list: List<any>;
} | {
    kind: ListEventKind.MemberUpdate;
    list: List<any>;
    member: Member<any>;
};
export declare class List<T> {
    /** @internal */
    array: Member<T>[];
    size: State<number>;
    /** @internal */
    protected listeners: ((event: ListEvent) => void)[];
    set(iterable: Iterable<T>): void;
    constructor(init?: Iterable<T>);
    get first(): Member<T> | undefined;
    get last(): Member<T> | undefined;
    static isList<T>(value: any): value is List<T> | ReadonlyList<T>;
    static isReadonly<T>(value: any): value is ReadonlyList<T>;
    [Symbol.iterator](): IterableIterator<Member<T>>;
    removeMember(member: Member<T>): number;
    replaceMember(member: Member<T>, value: T): number;
    append(value: T): Member<T>;
    prepend(value: T): Member<T>;
    dropLast(): Member<T> | undefined;
    dropFirst(): Member<T> | undefined;
    /** @escapes This function escapes the list and returns a plain array. The result is not reactive. */
    toArray(): T[];
    find(value: T): Member<T> | undefined;
    at(index: number): Member<T> | undefined;
    insertAt(index: number, value: T): Member<T>;
    removeAt(index: number): Member<T>;
    replaceAt(index: number, value: T): Member<T>;
    swap(a: Member<T>, b: Member<T>): void;
    swapBetween(i: number, j: number): void;
    moveBefore(before: Member<T>, member: Member<T>): void;
    moveTo(index: number, member: Member<T>): void;
    readonly(): ReadonlyList<T>;
    listen(listener: (event: ListEvent) => void): () => void;
    /** @internal */
    notify(event: ListEvent): void;
    join(joiner: string): State<string>;
    /** The sliced list will not react to changes in the original list. */
    slice(start?: number, end?: number): ReadonlyList<T>;
    reverse(): ReadonlyList<T>;
    /** Like `Array#map`, but function receives List Members instead of values. Returns a new ReadonlyList. */
    each<U>(modifier: (value: Member<T>) => U): ReadonlyList<U>;
    sort(compareFn: (a: T, b: T) => number): List<T>;
    filter(filterFn: (value: T, index: number) => boolean): List<T>;
}
export declare class ReadonlyList<T> {
    /** @internal */
    source: List<T>;
    size: ReadonlyState<number>;
    listen: (listener: (event: ListEvent) => void) => () => void;
    at: (index: number) => ReadonlyMember<T> | undefined;
    reverse: () => ReadonlyList<T>;
    slice: (start: number, end: number) => ReadonlyList<T>;
    toArray: () => T[];
    join: (joiner: string) => ReadonlyState<string>;
    constructor(source: List<T>);
    [Symbol.iterator](): Generator<ReadonlyMember<T>, void, unknown>;
    each<U>(modifier: (value: ReadonlyMember<T>) => U): ReadonlyList<U>;
    readonly(): ReadonlyList<T>;
}
export {};
