import type { Tag } from "./lib/tags.ts";
import type { EmptyElements } from "./lib/emptyElements.ts";
import type { Attributes } from "./attributes.ts";
import { Falsy } from "./util.ts";
import { ReadonlyState } from "./state.ts";
import * as Context from "./context.internal.ts";
import { List, ReadonlyList } from "./list.ts";
export type NonEmptyElement = Exclude<Tag, EmptyElements>;
export type HyperTextNode = string;
export declare class HyperHTMLStringNode {
    htmlString: string;
    constructor(htmlString: string);
}
export declare class HyperNode<T extends Tag> {
    tag: T;
    attrs: Attributes<T>;
    children: HyperNodeish[];
    constructor(tag: T, attrs: Attributes<T>, children: HyperNodeish[]);
}
export type HyperChild<T extends Tag> = HyperNode<T> | HyperHTMLStringNode | HyperTextNode;
export type HyperNodeish = HyperChild<any> | Falsy | ReadonlyState<HyperNodeish> | ReadonlyList<HyperNodeish> | List<HyperNodeish> | Context.Provider<any> | Context.Consumer<any>;
export declare const isHyperChild: (n: any) => n is HyperChild<any>;
export declare const isHyperNodeish: (x: any) => x is HyperNodeish;
export declare function normaliseParams<T extends Tag>(props?: Attributes<T> | HyperNodeish, childNodes?: HyperNodeish[]): {
    attrs: Attributes<T>;
    children: HyperNodeish[];
};
export declare function h<Tag extends NonEmptyElement, Attrs extends Attributes<Tag>>(elem: Tag, props?: Attrs | Falsy): HyperNode<Tag>;
export declare function h<Tag extends NonEmptyElement>(elem: Tag, ...children: HyperNodeish[]): HyperNode<Tag>;
export declare function h<Tag extends NonEmptyElement, Attrs extends Attributes<Tag>>(elem: Tag, props: Attrs, ...children: HyperNodeish[]): HyperNode<Tag>;
export declare function h<Tag extends NonEmptyElement, Attrs extends Attributes<Tag>>(elem: Tag, props?: Attrs | HyperNodeish | Falsy, ...children: HyperNodeish[]): HyperNode<Tag>;
export declare function h<Tag extends EmptyElements, Attrs extends Attributes<Tag>>(elem: Tag, props?: Attrs | HyperNodeish | Falsy): HyperNode<Tag>;
export declare function trust(html: string): HyperHTMLStringNode;
