import { isFalsy, isNonNullable } from "./util.js";
import { ReadonlyState, State } from "./state.js";
import { Context as Context2 } from "./context.js";
import { List } from "./list.js";
export class HyperHTMLStringNode {
    htmlString;
    constructor(htmlString) {
        this.htmlString = htmlString;
    }
}
export class HyperNode {
    tag;
    attrs;
    children;
    constructor(tag, attrs, children) {
        this.tag = tag;
        this.attrs = attrs;
        this.children = children;
    }
}
export const isHyperChild = (n) => n instanceof HyperNode ||
    n instanceof HyperHTMLStringNode ||
    typeof n === "string" ||
    ReadonlyState.isState(n) ||
    List.isList(n);
export const isHyperNodeish = (x) => isHyperChild(x) || isFalsy(x) || State.isState(x) || Context2.isContext(x);
export function normaliseParams(props, childNodes) {
    const [attrs, children] = isHyperNodeish(props)
        ? [{}, [props, ...(childNodes || [])]]
        : [props || {}, childNodes || []];
    return { attrs, children };
}
export function h(tag, props, ...childNodes) {
    const { attrs, children } = normaliseParams(props, childNodes);
    return new HyperNode(tag, attrs, children.filter(isNonNullable));
}
export function trust(html) {
    return new HyperHTMLStringNode(html);
}
