import { HyperHTMLStringNode } from "../node.js";
import { EmptyElements } from "../lib/emptyElements.js";
import { ReadonlyState, State } from "../state.js";
import { escapeAttr, escapeTextNode, isFalsy } from "../util.js";
import { List } from "../list.js";
import * as Context from "../context.internal.js";
function eventListeners(attrs) {
    // noop
    return false;
}
function aria(attrs) {
    if (!attrs)
        return "";
    return Object.entries(attrs)
        .map(([attr, v]) => {
        const key = `aria-${attr}`;
        let value = ReadonlyState.isState(v) ? v.value : v;
        // @ts-expect-error TODO: some aria properties will allow booleans in the future
        if (value === true)
            value = "";
        if (!value)
            return "";
        return `${key}="${value}"`;
    })
        .join(" ");
}
function attrifyHTML(attrs) {
    return Object.entries(attrs)
        .map(([k, v]) => {
        const attr = k;
        if (attr === "on")
            return eventListeners(attrs[attr]);
        if (attr === "aria") {
            const value = attrs[attr];
            return aria(value);
        }
        if (attr === "ref")
            return false;
        let value = v;
        if (ReadonlyState.isState(value))
            value = value.value;
        if (value === true)
            return attr;
        if (value === "")
            return attr;
        if (Array.isArray(value))
            return `${attr}="${escapeAttr(value.filter(Boolean).join(" "))}"`;
        if (value)
            return `${attr}="${escapeAttr(String(value))}"`;
    })
        .filter(Boolean)
        .join(" ");
}
function toHTML(node, environment) {
    if (node instanceof Context.Provider) {
        const registry = new Map(environment.registry);
        registry.set(node.contextId, node);
        return toHTML(node.contextualChild, { ...environment, registry });
    }
    if (node instanceof Context.Consumer) {
        const ctx = environment.registry.get(node.contextId);
        if (!ctx)
            throw new Error(`Requested context for (id: ${String(node.contextId).slice(7, -1)}) not found. Was the Context Provider used?`);
        return toHTML(node.renderWithContext(ctx.contextValue), environment);
    }
    if (isFalsy(node))
        return "";
    if (typeof node === "string")
        return escapeTextNode(node);
    if (node instanceof HyperHTMLStringNode)
        return node.htmlString;
    if (State.isState(node))
        return renderHTML(node.value);
    if (List.isList(node))
        return node.toArray().map(renderHTML).join("");
    let stringified = "<" + node.tag;
    const attr = attrifyHTML(node.attrs);
    if (attr)
        stringified += " " + attr;
    if (EmptyElements.has(node.tag))
        stringified += " />";
    else if (node.children.length)
        stringified += ">" + node.children.map(child => toHTML(child, environment)).join("") + `</${node.tag}>`;
    else
        stringified += `></${node.tag}>`;
    return stringified;
}
export function renderHTML(node) {
    return toHTML(node, { registry: new Map() });
}
