export type Subscriber = (value: any) => void;
type ComposedStateValue<Obj extends Record<string, State>> = {
    [key in keyof Obj]: Obj[key]["value"];
};
export declare class ReadonlyState<T = any> {
    private source?;
    protected subscribers: Subscriber[];
    protected state: {
        value: T;
    };
    constructor(value: T, source?: ReadonlyState<T> | undefined);
    get value(): T;
    to<U>(transformer: (t: T) => U): ReadonlyState<U>;
    pipe(state: State<T>): void;
    static isState(x: any): x is State | ReadonlyState;
    /**
     * Compose multiple states into a single state
     */
    static compose<RefMap extends {
        [k: string]: State;
    }>(refs: RefMap): ReadonlyState<ComposedStateValue<RefMap>>;
    if(predicate: (value: T) => boolean): ReadonlyState<T | null>;
    debounce(ms: number): ReadonlyState<T>;
    listen(listener: (value: T) => void): () => void;
}
export declare class State<T = any> extends ReadonlyState<T> {
    constructor(value: T);
    set(next: T): T;
    setWith(updater: (value: T) => T): T;
    readonly(): ReadonlyState<T>;
}
export {};
